Attribute VB_Name = "modLexer"
'---------------------------------------------------------------------------------------
' Module    : modLexer
' Author    :
' Date      : 11.04.2012
' Purpose   :  
'---------------------------------------------------------------------------------------
Option Explicit

Enum Tok_Types
    NONE = 0
    LSTRING = 1
    DIGIT = 2
    QSTRING = 3
    LPARM = 4
    RPARM = 5
    LVARIABLE = 6
    COMMENT = 7
    KEYWORD = 8
    DEL = 9
    EOL = 10
    EOP = 11
    
    VAR_INT = 12
    VAR_Long = 13
    VAR_BYTE = 14
    VAR_FLOAT = 15
    VAR_str = 16
    
    LOCAL_VAR = 17
    
    FUNC = 18
    LEX_RETURN = 19
    
    LEX_IF = 20
    
    LCurly = 30
    RCurly = 31
    
    LBracket = 32
    RBracket = 33
    
    
    
    
End Enum

'------------------------------------------------------------
'  
Type LEX_TABLE
  TOKEN As String '
  Class As String ' 
  ID_CLASS As Long 'ID 
  STRING_NUMBER As Long '     
  LOCAL_STRING_POS As Long '   
  GLOBAL_STRING_POS As Long '   
  Area As String ' 
  FILE_NAME As String ' 
End Type
'------------------------------------------------------------
 
Private CharPos As Long 'Current processing char
Private Source As String 'Source to scan
Private TOKEN As String 'Hold the returned Token
Private TokType As Tok_Types 'Hold the Token Type
Private Keywords() As String 'Just hold some Keywords
Public LexTable() As LEX_TABLE '  
Public LexPointer As Long '    

Public LinePos As Long '     
Public TokenStrPos As Long '  
Public Area As String ' 
Public MediumCode As String ' 


'---------------------------------------------------------------------------------------
' Procedure : LexerInit
' Author    :
' Date      : 11.04.2012
' Purpose   :    
'---------------------------------------------------------------------------------------
Sub LexerInit()
   LexPointer = 1
   CharPos = 1
   LinePos = 1
  
   
   Erase Keywords
   Erase LexTable
   
   ReDim Keywords(0) As String
   ReDim LexTable(0) As LEX_TABLE
  
  
   KeywordAdd "BYTE"
   KeywordAdd "INT"
   KeywordAdd "LONG"
   KeywordAdd "FLOAT"
   KeywordAdd "STR"
   
   KeywordAdd "LOCAL"
   KeywordAdd "CONST"
   KeywordAdd "TYPE"
   
   
   
   KeywordAdd "FUNC"
   KeywordAdd "IF"
   KeywordAdd "RETURN"
   KeywordAdd "LOOP"

   KeywordAdd "FOR"
   KeywordAdd "DESTROY"
   


End Sub

 
Sub LexerNextFile()
   LexPointer = 1
   CharPos = 1
   LinePos = 1
  
End Sub

 
 
 
 
'---------------------------------------------------------------------------------------
' Procedure : RunLexer
' Author    :
' Date      : 11.04.2012
' Purpose   :   
'---------------------------------------------------------------------------------------
'
Function RunLexer(ByVal SourceCode As String, ByVal FileName As String) As Long
 ' Source = Source & SourceCode & vbCrLf
  Source = SourceCode & vbCrLf
  Area = "GLOBAL"
    Do
        'Loop until we hit the EOP token-type
       GetToken
       Call LexTableAdd(TOKEN, GetStrToken(TokType), TokType, LinePos, TokenStrPos, CharPos, FileName)
       Debug.Print TOKEN; CharPos
       If GetStrToken(TokType) = "EOL" Then LinePos = LinePos + 1: TokenStrPos = 0
    Loop Until (TokType = EOP)
    
End Function

 
Private Function LexTableAdd(ByVal TOKEN As String, _
                              ByVal Class As String, _
                              ByVal TOK_TYPE As Long, _
                              ByVal LinePos As String, _
                              ByVal TokenStrPos As Long, _
                              ByVal GlobalTokenStrPos As Long, _
                              ByVal FileName As String)
                      
                              
 ReDim Preserve LexTable(UBound(LexTable) + 1)
 LexTable(UBound(LexTable)).TOKEN = TOKEN
 LexTable(UBound(LexTable)).Class = Class
 LexTable(UBound(LexTable)).ID_CLASS = TOK_TYPE
 LexTable(UBound(LexTable)).STRING_NUMBER = LinePos
 LexTable(UBound(LexTable)).FILE_NAME = FileName
 
 If Area = "LOCAL" Then
   Area = TOKEN
 End If
 
 If TOK_TYPE = FUNC Then
    Area = "LOCAL"
 End If
  
 LexTable(UBound(LexTable)).Area = Area
  
 
 If Class = "Quote_String" Then
    LexTable(UBound(LexTable)).LOCAL_STRING_POS = (TokenStrPos - Len(TOKEN)) - 1   '  
    LexTable(UBound(LexTable)).GLOBAL_STRING_POS = (GlobalTokenStrPos - Len(TOKEN)) '- 1
 Else
    LexTable(UBound(LexTable)).LOCAL_STRING_POS = (TokenStrPos - Len(TOKEN)) + 1   '  
    LexTable(UBound(LexTable)).GLOBAL_STRING_POS = (GlobalTokenStrPos - Len(TOKEN)) ' + 1
 End If
 
 
End Function
 
 
Private Function GetStrToken(iTokT As Tok_Types) As String
    'All this does is return the string name of a Token Type ID
    Select Case iTokT
        Case NONE
            GetStrToken = "Nothing"
        Case LSTRING
            GetStrToken = "String"
        Case DIGIT
            GetStrToken = "Number"
        Case QSTRING
            GetStrToken = "Quote_String"
        Case LPARM
            GetStrToken = "LPARM"
        Case RPARM
            GetStrToken = "RPARM"
        Case LVARIABLE
            GetStrToken = "Variable"
        Case COMMENT
            GetStrToken = "Comment"
        Case KEYWORD
            GetStrToken = "IDENT"
        Case DEL
            GetStrToken = "Delimiter"
        Case EOL
            GetStrToken = "EOL"
        Case EOP
            GetStrToken = "EOP"
            
        Case VAR_INT
            GetStrToken = "Var Int"
            
        Case VAR_Long
            GetStrToken = "Var Long"
            
        Case VAR_BYTE
            GetStrToken = "Var Byte"
        
        Case VAR_FLOAT
            GetStrToken = "Var Float"
        Case VAR_str
            GetStrToken = "Var String"
            
        Case LOCAL_VAR
            GetStrToken = "Local var"
            
        Case FUNC
           GetStrToken = "Function"
           
        Case LEX_IF
           GetStrToken = "IF"
            
        Case LCurly
            GetStrToken = "LCurly"
            
        Case RCurly
            GetStrToken = "RCurly"
  
            
        Case LBracket
           GetStrToken = "LBracket"
        
        Case RBracket
           GetStrToken = "RBracket"
            
        Case Else
            GetStrToken = "UNKOWN"
            
            
    End Select
    
End Function

Private Function IsKeyword(Key As String) As Boolean
Dim x As Integer
    'Return true or false if we found a Keyword
    IsKeyword = False
    For x = 0 To UBound(Keywords)
        If Keywords(x) = UCase(Key) Then IsKeyword = True: Exit For
    Next x
End Function

Public Function isAlpha(c As String) As Boolean
    ' a-z  A-Z
    isAlpha = UCase(c) >= "A" And UCase(c) <= "Z"
End Function

Public Function isWhite(c As String) As Boolean
    '
    isWhite = (c = " ") Or (c = vbTab)
End Function

Public Function isDigit(c As String) As Boolean
    '
    isDigit = (c >= "0") And (c <= "9")
End Function

Function IsDelim(c As String) As Boolean
    '
    If InStr(" ,;<>+-/*%^=[]()&{}|", c) Or c = vbCr Then IsDelim = True
End Function

Sub INC(Optional nMove As Integer = -1)
    If (nMove <> -1) Then
        CharPos = CharPos + nMove
        TokenStrPos = TokenStrPos + nMove
    Else
        CharPos = CharPos + 1
        TokenStrPos = TokenStrPos + 1
        
    End If
End Sub

Public Function KeywordAdd(ByVal Name As String)
 ReDim Preserve Keywords(UBound(Keywords) + 1)
 Keywords(UBound(Keywords)) = Name
End Function

'---------------------------------------------------------------------------------------
' Procedure : GetToken
' Author    :
' Date      : 11.04.2012
' Purpose   :    
'---------------------------------------------------------------------------------------
'
Sub GetToken()
 '   
 '      ,   
     
    TOKEN = ""
    TokType = NONE
    
    'If we are over the length of the souce we are at the end of the program
    If (CharPos > Len(Source)) Then
        ' 
        TokType = EOP
        TOKEN = Chr(11)
        Exit Sub
    End If
    
    ' 
    Do While (CharPos <= Len(Source) And (isWhite(Mid(Source, CharPos, 1))))
       INC
        If CharPos > Len(Source) Then
            TokType = EOL
            Exit Sub
        End If
    Loop
    
    '   
    If Mid(Source, CharPos, 1) = vbCr Then
        INC 2
        TOKEN = vbCr
        TokType = EOL
        Exit Sub
    End If
    
    
   If (Mid(Source, CharPos, 2)) = "//" Then
     INC
     INC
     
       While Mid(Source, CharPos, 1) <> vbCr
            TOKEN = TOKEN + Mid(Source, CharPos, 1)
            INC
            
            If (CharPos > Len(Source)) Then Exit Sub
        Wend
        TokType = COMMENT
        Exit Sub
   End If
    
    
    
    '   ,;<>+-/*%^=[]()&
    If IsDelim(Mid(Source, CharPos, 1)) Then
    
   
        TOKEN = TOKEN + Mid(Source, CharPos, 1)
        INC
        '    ()
        If (TOKEN = "(") Then
            TokType = LPARM
        ElseIf (TOKEN = ")") Then
            TokType = RPARM
            
        ElseIf (TOKEN = "{") Then
            TokType = LCurly
            
        ElseIf (TOKEN = "}") Then
            TokType = RCurly
            
        ElseIf (TOKEN = "[") Then
            TokType = LBracket
        
        ElseIf (TOKEN = "]") Then
            TokType = RBracket
            
            
            
            
        Else
            '  
            TokType = DEL
        End If
 
       
    'This part I added for my own purpose to ident a variable
    ElseIf Mid(Source, CharPos, 1) = "$" Then
        INC
        TOKEN = "$"
        If isAlpha(Mid(Source, CharPos, 1)) Then
            While Not IsDelim(Mid(Source, CharPos, 1))
                TOKEN = TOKEN + Mid(Source, CharPos, 1)
                INC
            Wend
        End If
        TokType = LVARIABLE
        Exit Sub
    
    'Checks for only Alpha strings
    ElseIf isAlpha(Mid(Source, CharPos, 1)) Then
        While Not IsDelim(Mid(Source, CharPos, 1))
            TOKEN = TOKEN + Mid(Source, CharPos, 1)
            INC
            TokType = LSTRING
        Wend
        'Check if we have a keyword other wise it's a LSTRING
        If IsKeyword(TOKEN) Then
           TokType = KEYWORD
         
          Select Case UCase$(TOKEN)
                   Case "INT": TokType = VAR_INT
                   Case "LONG": TokType = VAR_Long
                   Case "FLOAT": TokType = VAR_FLOAT
                   Case "BYTE": TokType = VAR_BYTE
                   Case "STR": TokType = VAR_str
                    
                   Case "LOCAL": TokType = LOCAL_VAR
                   Case "FUNC": TokType = FUNC
                   
                   Case "IF": TokType = LEX_IF
                     
          End Select
          
         
         
         End If
         
         
         
         
        'Check for digits
    ElseIf isDigit(Mid(Source, CharPos, 1)) Then
        While Not IsDelim(Mid(Source, CharPos, 1))
            TOKEN = TOKEN + Mid(Source, CharPos, 1)
            If CharPos > Len(Source) Then Exit Sub
            INC
        Wend
        TokType = DIGIT
        Exit Sub
    
    '    "hello world"
    ElseIf Mid(Source, CharPos, 1) = Chr(34) Then
        INC
        While Mid(Source, CharPos, 1) <> Chr(34) And Mid(Source, CharPos, 1) <> vbCr
            TOKEN = TOKEN + Mid(Source, CharPos, 1)
            INC
            If CharPos > Len(Source) Then
                Exit Sub
            End If
        Wend
        If Mid(Source, CharPos, 1) = vbCr Then
            TOKEN = Chr(0)
            TokType = EOP
            Exit Sub
        End If
        INC
        TokType = QSTRING
        Exit Sub
    '---------------------------------------------------
    ElseIf (Mid(Source, CharPos, 1) = "'") Then
        While Mid(Source, CharPos, 1) <> vbCr
            TOKEN = TOKEN + Mid(Source, CharPos, 1)
            INC
            
            If (CharPos > Len(Source)) Then Exit Sub
        Wend
        TokType = COMMENT
    Else
        'Beep
        TOKEN = "PHASE_ERROR"
         TokType = EOP
    End If
    
End Sub

